<?php

// Block direct access
if (!defined('ABSPATH')) { exit; }

// Add menu page
function error_log_viewer_menu() {
    add_management_page('error_log File Viewer', 'error_log', 'manage_options', 'error_log', 'error_log_viewer_page');
}
add_action('admin_menu', 'error_log_viewer_menu');

// On POST request this truncates the error_log file
add_action('wp_ajax_empty_error_log', 'empty_error_log_ajax_handler');
function empty_error_log_ajax_handler() {
  // Check permissions
  if (!current_user_can('manage_options')) {
    wp_send_json_error('Insufficient permissions.');
  }

  // Path to error_log file
  $config_path = ABSPATH . 'error_log';

  // Empty the error log file
  file_put_contents($config_path, '');

  wp_send_json_success('Error log emptied successfully.');
}

// Settings page
function error_log_viewer_page() {
    if (!current_user_can('manage_options')) {
        wp_die('You do not have sufficient permissions to access this page.');
    }
    
    // Check the active wp-admin page and load these files only on the plugin settings page
    $screen = get_current_screen();
       if ( $screen->id === 'tools_page_error_log'){
            wp_enqueue_style('tools_page_error_log-admin_page', plugins_url('css/settings-page.css', __FILE__));
            wp_enqueue_script( 'error_log_viewer-js', plugin_dir_url( __FILE__ ) . '/js/error_log_download.js', array( 'jquery' ), '1.0', true );
            wp_enqueue_script('error_log_empty-js', plugin_dir_url(__FILE__) . '/js/error_log_empty.js', array('jquery'), '1.1', true);
            wp_enqueue_script('error_log_expand-js', plugin_dir_url(__FILE__) . '/js/error_log_expand.js', array('jquery'), '1.2', true);
    }

    // path to error_log file
    $config_path = ABSPATH . 'error_log';
	
	// Get the size of the error_log file
	$file_size = filesize($config_path);

    // Convert the file size to a human-readable format
    $human_readable_size = size_format($file_size);

    // Get the number of lines the user wants to view
    $num_lines = 25; // Default value
    if (isset($_POST['num_lines']) && is_numeric($_POST['num_lines'])) {
        $num_lines = intval($_POST['num_lines']);
    }

    // Get the contents of the error_log file
    $content = file_get_contents($config_path);

    // Split the contents into an array of lines
    $lines = explode("\n", $content);

    // Get only the last N lines
    $lines = array_slice($lines, -$num_lines);

    // Convert the array of lines back to a string
    $content = implode("\n", $lines);
    ?>
    <div id="pluginsclub-cpanel">
					<div id="pluginsclub-cpanel-header">
			<div id="pluginsclub-cpanel-header-title">
				<div id="pluginsclub-cpanel-header-title-image">
<h1><a href="http://plugins.club/" target="_blank" class="logo"><img src="<?php echo plugins_url('images/pluginsclub_logo_black.png', __FILE__) ?>" style="height:27px"></a></h1></div>
				<div id="pluginsclub-cpanel-header-title-image-sep">
				</div>
<div id="pluginsclub-cpanel-header-title-nav">
	<?php
// Get our API endpoint and from it build the menu
$plugins_club_api_link = 'https://api.plugins.club/list_of_wp_org_plugins.php';
$remote_data = file_get_contents($plugins_club_api_link);
$menuItems = json_decode($remote_data, true);

foreach ($menuItems as $menuItem) :
    $isActive = isset($_GET['page']) && ($_GET['page'] === $menuItem['page']);
    $activeClass = $isActive ? 'active' : '';
    $isInstalled = function_exists($menuItem['check_function']) && function_exists($menuItem['check_callback']);
    $name = $menuItem['name'];
    if (!$isInstalled) {
        $name = ' <span class="dashicons dashicons-plus-alt"></span> '.$name;
    } else {
        $name .= ' <span class="dashicons dashicons-plugins-checked"></span>';
    }
?>
    <div class="pluginsclub-cpanel-header-nav-item <?php echo $activeClass; ?>">
        <?php if ($isInstalled) : ?>
            <a href="<?php echo $menuItem['url']; ?>" class="tab"><?php echo $name; ?></a>
        <?php else : ?>
            <a href="<?php echo $menuItem['fallback_url']; ?>" target="_blank" class="tab"><?php echo $name; ?></a>
        <?php endif; ?>
    </div>
<?php endforeach; ?>

<div id="pluginsclub-cpanel-header-title-image-sep">
				</div>
      
			</div>
		</div>
		
	</div>	
		  <div class="">

				<div id="pluginsclub-cpanel-admin-wrap" class="wrap">
			<h1 class="pluginsclub-cpanel-hide"><?php echo esc_html(get_admin_page_title()); ?> <a href="<?php echo esc_url(site_url('/wp-admin/admin-ajax.php?action=download_error_log')); ?>" class="button-primary" id="download-error-log">Download error_log</a></h1>

			<form id="pluginsclub-cpanel-form" method="post">
<h2>
    <?php echo esc_html(get_admin_page_title()); ?>
    <sup><span class="file-size">(File Size: <?php echo $human_readable_size; ?>)</span></sup>
<?php
$raw = isset($_GET['raw']) ? $_GET['raw'] : '';
if ($raw === 'true') {
    echo '<a class="button-secondary" href="' . esc_url(admin_url('tools.php?page=error_log')) . '"><span class="dashicons dashicons-editor-table"></span> View Table</a>';
} else {
    echo '<a class="button-secondary" href="' . esc_url(admin_url('tools.php?page=error_log')) . '&raw=true"><span class="dashicons dashicons-text"></span> View Raw Log</a>';

}    
?>
    
    
    
    
    
    <a href="<?php echo esc_url(site_url('/wp-admin/admin-ajax.php?action=download_error_log')); ?>" class="button-primary" id="download-error-log">
        <span class="button-icon dashicons dashicons-download"></span> Download error_log
    </a>
    <button class="button-secondary" id="empty-error-log">
        <span class="button-icon dashicons dashicons-trash"></span> Empty Error Log
    </button>
</h2>

		<div class="pluginsclub-cpanel-sep"></div></br>
		<p>
		    

		    
			<form method="post">
    <label for="num_lines">Lines:</label>
    <select id="num_lines" name="num_lines">
        <option value="25" <?php selected($num_lines, 25); ?>>25</option>
        <option value="50" <?php selected($num_lines, 50); ?>>50</option>
        <option value="100" <?php selected($num_lines, 100); ?>>100</option>
        <option value="250" <?php selected($num_lines, 250); ?>>250</option>
        <option value="500" <?php selected($num_lines, 500); ?>>500</option>
        <option value="1000" <?php selected($num_lines, 1000); ?>>1000</option>
    </select>
<?php
$raw = isset($_GET['raw']) ? $_GET['raw'] : '';
if ($raw === 'true') {

    
} else {
?>
    <label for="error_type">Types:</label>
    <select id="error_type" name="error_type">
        <option value="all" <?php selected($selected_error_type, 'all'); ?>>All</option>
        <option value="PHP Fatal error" <?php selected($selected_error_type, 'PHP Fatal error'); ?>>PHP Fatal error</option>
        <option value="WordPress database error" <?php selected($selected_error_type, 'WordPress database error'); ?>>Database error</option>
        <option value="PHP Warning" <?php selected($selected_error_type, 'PHP Warning'); ?>>PHP Warning</option>
    </select>

    <label for="selected_date">Select Date:</label>
    <input type="date" id="selected_date" name="selected_date" value="<?php echo isset($_POST['selected_date']) ? $_POST['selected_date'] : ''; ?>">

<?php
}
?>
    <input type="submit" value="Update and Filter" class="button-secondary">
</form>

    
    </p>
<?php
// Handle form submission
if (isset($_POST['selected_date'])) {
    $selected_date = $_POST['selected_date'];
    // Format the selected date as d-m-Y
    $formatted_date = date('d-m-Y', strtotime($selected_date));
} else {
    $selected_date = null;
    $formatted_date = null;
}


// Handle form submission
if (isset($_POST['error_type'])) {
    $selected_error_type = $_POST['error_type'];
} else {
    $selected_error_type = 'all';
}
?>
    <div class="wrap">
    <?php
	// If file does not exist or is empty display a message suggesting to enable error logging
    if (!file_exists($config_path) || filesize($config_path) == 0) {
    echo "
    <div class='notice notice-error inline'>
	<p>Looks like the error_log file does not exist or is empty. <b>Are you sure error logging is enabled?</b></p>
	<p>Using the <a href='https://wordpress.org/plugins/enable-wp-debug-toggle/' target='_blank'>WP_DEBUG Toggle plugin</a> you can easilly toggle WP_DEBUG ON or OFF from the wp-admin bar and update the wp-config.php file.</p>
</div>";
    return;
	}
    ?>
    
    <style>
    pre {
    white-space: -moz-pre-wrap; /* Mozilla, supported since 1999 */
    white-space: -pre-wrap; /* Opera */
    white-space: -o-pre-wrap; /* Opera */
    white-space: pre-wrap; /* CSS3 - Text module (Candidate Recommendation) http://www.w3.org/TR/css3-text/#white-space */
    word-wrap: break-word; /* IE 5.5+ */
}
    </style>
    
    
<?php
$raw = isset($_GET['raw']) ? $_GET['raw'] : '';

if ($raw === 'true') {
    echo '<pre style="background-color:#f9f9f9; border: 1px solid #ddd; padding: 10px; font-size: 14px; overflow: auto;">';
    echo esc_html($content);
    echo '</pre>';
} else {
?>    
    <table class="wp-list-table widefat striped">
        <thead>
            <tr>
                <th>Date</th>
                <th>Type</th>
                <th>Error Message</th>
            </tr>
        </thead>
        <tbody>
        <?php
// Combine lines into rows
$rows = array();
$currentRow = array();
foreach ($lines as $line) {
    if (substr($line, 0, 1) === '[') {
        // Start of a new row
        if (!empty($currentRow)) {
            $rows[] = $currentRow;
        }
        $currentRow = array();
        // Split the line into Date and Type
        preg_match('/^\[(.*?)\]\s(.*?)\:\s/', $line, $matches);
        if (count($matches) === 3) {
            $currentRow['Date'] = $matches[1];
            $currentRow['Type'] = $matches[2];
            // Remove the timestamp and "PHP Fatal error" prefix from the error message
            $currentRow['Error'] = preg_replace('/^\[.*?\]\s.*?\:\s/', '', $line);
        } else {
            // If the line format is not as expected, treat it as a single column
            $currentRow['Date'] = '';
            $currentRow['Type'] = '';
            $currentRow['Error'] = $line;
        }
    } else {
        // Continue the current row
        $currentRow['Error'] .= "\n" . $line;
    }
}
// Add the last row
if (!empty($currentRow)) {
    $rows[] = $currentRow;
}

        // Filter rows based on the selected error type
        $filteredRows = array();
        if ($selected_error_type === 'all') {
            $filteredRows = $rows;
        } else {
            foreach ($rows as $row) {
                if (strpos($row['Type'], $selected_error_type) !== false) {
                    $filteredRows[] = $row;
                }
            }
        }

        // Loop through each row
        foreach ($filteredRows as $row) {
                // Retrieve the date from the row
    $rowDate = date('d-m-Y', strtotime($row['Date']));
            
            
            // Highlight the row for each type
if ($row['Type'] === 'PHP Fatal error') {
    $rowColor = '#E30B5C';
} elseif ($row['Type'] === 'WordPress database error') {
    $rowColor = '#FFC300';
} else {
    $rowColor = '';
}

if (empty($selected_date) || $rowDate === $formatted_date) {

            // Output the row as table cells
            echo '<tr>';
            // Display the Date column in a human-readable format
    $formattedDate = date('d M Y H:i:s T', strtotime($row['Date']));
    echo '<td> <span style=" white-space: nowrap;">' . $formattedDate . '</span></td>';
    
// Initialize the variable to store the plugin or theme name
$pluginThemeName = '';

// Extract the plugin name from the error line using regular expression
preg_match('/plugins\/([^\/]+)\//', $row['Error'], $pluginMatches);

// Extract the theme name from the error line using regular expression
preg_match('/themes\/([^\/]+)\//', $row['Error'], $themeMatches);

// Check if a plugin name match is found
if (isset($pluginMatches[1])) {
    $pluginSlug = $pluginMatches[1];

    // Get the plugin data from the plugin slug
    $pluginData = get_plugin_data(WP_PLUGIN_DIR . '/' . $pluginSlug . '/' . $pluginSlug . '.php');

    // Check if the plugin data exists and contains the name
    if ($pluginData && !empty($pluginData['Name'])) {
        $pluginThemeName = 'Plugin: ' . $pluginData['Name']; // Add "Plugin: " prefix
    } else {
        $pluginThemeName = 'Plugin: ' . $pluginSlug; // Use the plugin slug if the name is not available
    }

    $class = 'plugin'; // Set the class for plugins
    $backgroundColor = 'blue'; // Set the background color for plugins
}
// If no plugin name match is found, check if a theme name match is found
elseif (isset($themeMatches[1])) {
    $themeSlug = $themeMatches[1];

    // Get the theme object from the theme slug
    $theme = wp_get_theme($themeSlug);

    // Check if the theme object exists and contains the name
    if ($theme && !empty($theme->name)) {
        $pluginThemeName = 'Theme: ' . $theme->name; // Add "Theme: " prefix
    } else {
        $pluginThemeName = 'Theme: ' . $themeSlug; // Use the theme slug if the name is not available
    }

    $class = 'theme'; // Set the class for themes
    $backgroundColor = 'red'; // Set the background color for themes
}




            echo '<td> <span style=" white-space: nowrap;    padding: 5px 10px;
    vertical-align: middle;
    border-radius: 15px; background-color: ' . $rowColor . ';">' . nl2br(esc_html($row['Type'])) . '</span></td>';
// Limit the error text to 150 characters
        echo '<td><span class="' . $class . '" style=" white-space: nowrap; color: ' . $backgroundColor . ';">' . $pluginThemeName . '</span></br>';

    if (strlen($row['Error']) <= 150) {
    echo $row['Error'] . '</td>';
    } else {
        // Display truncated error with ellipsis
        $truncatedError = substr($row['Error'], 0, 147) . '...';
        echo '<span class="truncated-error">' . $truncatedError . '</span>';
        echo '<span class="expand-error" style="cursor: pointer; color: blue;">[Expand]</span>';
        echo '<span class="full-error" style="display: none;"><pre>' . $row['Error'] . '</pre></span>';
        echo '</td>';
    }
    echo '</tr>';
        }
        }
        ?>
        </tbody>
    </table>
    </div>
<?php
}
// else table end here
}

// Handle the Ajax request to download error_log file
function download_error_log_handler() {
  $config_path = ABSPATH . 'error_log';
  $content = file_get_contents($config_path);
  header("Content-Type: text/plain");
  header("Content-Disposition: attachment; filename=error_log.txt");
  echo $content;
  exit;
}
add_action("wp_ajax_download_error_log", "download_error_log_handler");